;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Numerical HP Counter (xkas version), by GreenHammerBro.
;;
;; Description: This generator display the boss HP in numerical (in numbers), it can
;; show up to four digits (0-9999HP), doesn't need custom layer 3 graphics.
;;
;; It does not need custom layer 3 graphics, since it only uses numbers and letters.
;;
;; Notes:
;; -The tile properties (YXPCCCTT) of the layer 3 status bar/HUD also effects the
;;  tiles of the Health counter.
;;
;; -4 8x8 tiles that the counter uses is ALWAYS used, even if there is less than 4 digits
;;  written (for example: xx30 HP, the xx before 30 is tile #$FC (blank tile) stored every
;;  frame).
;;
;; And remove their semicolons ";" before each opcode within a line.
;;
;; Uses first extra bit: NO
;;
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
!freeram_CurrentHP	= $7E0060	;>[2 bytes] the sprite's real HP.
!UseRecordEffect	= 01		;>0 = instantly update, 1 = slowly count as an animation
 !freeram_RecordEffect	= $7E0DC3	;>[2 bytes] increment/decrement effect, not real HP, just animation, not used if disabled.

!SBSize			= $02		;>$01 if 1 byte per 8x8 tile, $02 = 2 bytes per 8x8 tile (SSB patch).
!SBBossHPTextPos	= $7FA064	;>position of the text "HP:" (3 8x8s long), can be smw's HUD, or SSB's ram (but even numbers only).
!SBBossHPcoutPos	= $7FA06A	;>same as above but for numbers.

;^Do note that position is not automatically converted to sa-1, because its also used for the
;super status bar patch's freeram. Therefore, if you're using the old HUD (or 1 byte per 8x8),
;and use sa-1, use $6EF9 instead.

;If you want the display not to show the record delay
;effect, just scroll down to the instructions on how
;to make it use true HP (there are multiple you need to
;follow all).
;--------------------------
;DO NOT CHANGE THIS BELOW!
;--------------------------
if !UseRecordEffect == 0
	!CurrentHPNumbers = !freeram_CurrentHP
else
	!CurrentHPNumbers = !freeram_RecordEffect
endif

print "INIT ",pc	;\For some reason, sprite tool requires you to have this init, despite
	RTL		;/not being used
print "MAIN ",pc
        JSR ShowBossHP	;>run main code
        RTL		;>and done.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ShowBossHP:
	PHB	;\preserve data bank.
	PHK	;|
	PLB	;/
	PHX	;>protect sprite table (just in case).
;--------------------------------------------------------------------------------
;This code restore the status bar when 0HP , so if you have edited
;the status bar, or move this counter position, you should change this also.
;--------------------------------------------------------------------------------
	LDA #$FC				;>Load blank tile
	STA !SBBossHPTextPos			;>"H"
	STA !SBSize*1+!SBBossHPTextPos		;>"P"
	STA !SBSize*2+!SBBossHPTextPos		;>":"
	STA !SBBossHPcoutPos			;>Thousands place
	STA !SBSize*1+!SBBossHPcoutPos		;>Hundreds place
	STA !SBSize*2+!SBBossHPcoutPos		;>Tens place
	STA !SBSize*3+!SBBossHPcoutPos		;>ones place

	;LDA #$FC		;blank slots
	STA $0F09		;
	STA $0F0D		;
	STA $0F0E		;
	STA $0F0F		;

	LDA #$2E		;coin symbol and "X"
	STA $0F10		;
	LDA #$26		;
	STA $0F11		;
	LDA #$FC		;
	STA $0F12		;

	LDA #$3D		;the word "time"
	STA $0F0A		;
	LDA #$3E		;
	STA $0F0B		;
	LDA #$3F		;
	STA $0F0C		;

;	!smwtimerpos = $0F25	;If you put this meter on
;	LDA $0F31		;top of the timer, remove
;	STA !smwtimerpos	;the semicolon (";") on the
;	LDA $0F32		;very left side.
;	STA !smwtimerpos	;
;	LDA $0F33		;
;	STA !smwtimerpos	;

	LDA $0DBF			;restore coin counter routine
	CMP #$0A
	BCC .OneCoinDigit
	JSR HexDec2
	STY $0F13	
	STA $0F14	
	BRA .restoreSBdone
	.OneCoinDigit
	STA $0F14	
	LDA #$FC
	STA $0F13	
	.restoreSBdone
;--------------------------------------------------------------------
;The code below makes it so it doesn't store tiles if HP = 0.
;--------------------------------------------------------------------
	REP #$20				;\if HP not = 0, then skip return
	if !UseRecordEffect != 0		;|
		LDA !freeram_CurrentHP		;|
		ORA !freeram_RecordEffect	;|>if you are not using record, remove this.
	else					;|
		LDA !CurrentHPNumbers		;|
	endif					;|
	SEP #$20				;|
	BNE .no_zerohp				;|
	JMP .HPdone				;/
.no_zerohp
if !UseRecordEffect != 0
;--------------------------------------------------------------------;
;Record delay routine.
;--------------------------------------------------------------------;
	REP #$20
	LDA !freeram_RecordEffect	;\if both equal, don't inc/dec
	CMP !freeram_CurrentHP		;|
	BEQ .record_done		;/
	BCC .record_less_than		;>record is less than realHP, INC the record
;record_greater_than:
	LDA !freeram_RecordEffect
	SEC : SBC !freeram_CurrentHP	;\if the record is 15+ than the true HP, then
	CMP #$000F			;|drecement faster (subtract 15 per frame)
	BCS .fast_decrement		;/
	LDA !freeram_RecordEffect	;\
	DEC A				;|otherwise subtract 1 per frame
	BRA .store_to_record		;/
.fast_decrement
	LDA !freeram_RecordEffect	;\decrement by 15 per frame
	SEC : SBC #$000F			;|
	BRA .store_to_record		;/
.record_less_than
	LDA !freeram_CurrentHP			;\if true HP is 15+ greater than
	SEC : SBC !freeram_RecordEffect	;|record, then increment the
	CMP #$000F				;|record faster (add 15 per
	BCS .fast_increment			;/frame)
	LDA !freeram_RecordEffect		;\otherwise add 1 per frame
	INC A					;/
	BRA .store_to_record
.fast_increment
	LDA !freeram_RecordEffect	;\increment by 15 per frame
	CLC : ADC #$000F			;/
.store_to_record			;\done with record
	STA !freeram_RecordEffect	;|
.record_done				;|
	SEP #$20			;|
	BPL .skipreturnshort		;/
	JMP .HPdone
.skipreturnshort
endif
;----------------------------------------------------------------------------------
;Write the word 'HP:' on HUD, feel free to remove this if needed.
;----------------------------------------------------------------------------------
	LDA #$11			;>"H"
	STA !SBBossHPTextPos
	LDA #$19			;>"P"
	STA !SBSize*1+!SBBossHPTextPos
	LDA #$78			;>":"
	STA !SBSize*2+!SBBossHPTextPos
;----------------------------------------------------------------------------------
;Convert HP value to decimal digits.
;----------------------------------------------------------------------------------
	REP #$20
	LDA !CurrentHPNumbers			;\if boss HP less than 10,000
	CMP #$2710				;|then show numbers
	BCC .normal_hp				;/
	LDA #$2727				;\otherwise display "----"
	STA $0000				;|to not show glitched HP
	STA $0000+2				;/counter if > 9999 and return
	SEP #$20
	JMP .Scratchram2tiles
.normal_hp
	LDA !CurrentHPNumbers		;these are digit display.
	JSR HexDec16to311		
	STY $0000+2			;>3rd digit (remove if wanted 1 digit)
	STX $0000+3			;>4th digit
	SEP #$20
	JSR HexDec2
	STA $0000+1			;>2nd digit (remove if wanted 2 digits)
	STY $0000			;>1st digit (remove if wanted 3 digits)
	REP #$20
	LDA !CurrentHPNumbers		;\if less than 10, one digit
	CMP #$000A			;|
	BCC .onedigit			;/
	CMP #$0064			;\if less than 100, two digit
	BCC .twodigit			;/
	CMP #$03E8			;\if less than 1000, three digit
	BCC .threedigit			;/
	BRA .dont_hide_digit		;>if 4 digit, then no hides.
;----------------------------------------------
;These hide digits to remove "leading zeros".
;----------------------------------------------
.onedigit
	SEP #$20
	LDA #$FC			;\hide first, second and third digit.
	STA $0000			;|(remove if only wanted 1 digit shown)
	STA $0000+1			;|
	STA $0000+2			;/
	JMP .Scratchram2tiles
.twodigit
	SEP #$20
	LDA #$FC			;\hide first and second digit.
	STA $0000			;|(remove if only wanted 2 digit shown)
	STA $0000+1			;|
	JMP .Scratchram2tiles		;/>but "SEP #$20" and "JMP .HPdone" here.
.threedigit
	SEP #$20			;\hide only the first digit.
	LDA #$FC			;|(remove if only wanted 3 digits)
	STA $0000			;|
	JMP .Scratchram2tiles		;/>but "SEP #$20" and "JMP .HPdone" here.
.dont_hide_digit
	SEP #$20
.Scratchram2tiles

;--------------------------------------------------------------------------------
;Scratch ram to store digits to ASSB (current HP), remove semicolons if using it.
;--------------------------------------------------------------------------------
	LDA $0000			;\\first digit (thousands)
	STA !SBBossHPcoutPos		;|/
	LDA $0001			;|\second digit (hundreds)
	STA !SBSize*1+!SBBossHPcoutPos	;|/
	LDA $0002			;|\third digit (tens)
	STA !SBSize*2+!SBBossHPcoutPos	;|/
	LDA $0003			;|\fourth digit (ones)
	STA !SBSize*3+!SBBossHPcoutPos	;//

.HPdone
	PLX			;>end protect sprite table.
	PLB			;>and index table (just in case)
	RTS
;-------------------------------------------------;
;hex-dec converter, change this at your own risk  ;
;-------------------------------------------------;
HexDec16to311:
	STA $4204		;\hexdec for the last two digits.
	LDY #$0A		;|
	STY $4206		;|
	JSR .Waste24Cycles	;|
	LDX $4216		;|
	LDA $4214		;|
	STA $4204		;|
	LDY #$0A		;|
	STY $4206		;|
	JSR .Waste24Cycles	;|
	LDY $4216		;|
	LDA $4214		;/
	RTS

.Waste24Cycles
	JSR .Return
.Return
	RTS

HexDec2:
	LDY #$00	;\hexdec for the first two digits.
.loops			;|(hundreds and thousands place)
	CMP #$0A	;|
	BCC .outloop	;|
	SBC #$0A	;|
	INY		;|
	BRA .loops	;/
.outloop
	RTS