;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Percent HP Bar (generators version), by GreenHammerBro.
;;
;; Description: This generator, will display HP meter as numerical value and as a bar in
;; percent, like in pokemon.
;;
;; Notes:
;;
;; -If you want to change not to use the record effect, its sfx, or any options, just
;;  scroll down and you find the instructions on how to do so.
;;
;; -This code will consume A LOT OF TILES ON THE STATUS BAR!! If you are running low on
;;  tile space, remove the code below titled "This code will display the sprite's numerical
;;  max HP on status bar" to remove max HP display. Or better yet, use ASSB.
;;
;; -When editing this asm file, do note that xkas has a bug with the math statements on a number always
;;  pick the largest size possible, for example: LDA !Define+3 would assemble as LDA.l !Define+3. This
;;  can cause crash due to the last (or 2) byte being read as opcodes and reading the instructions at a
;;  shifted position, causing the game to crash. Use .b and .w to force the size.
;;
;; Uses first extra bit: NO
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


!freeram_CurrentHP	= $7E0060	;>[2 bytes], the sprite's current HP.
!freeram_MaxHP		= $7E0062	;>[2 bytes], the sprite's max HP.
!UseRecordEffect	= 1		;>0 = instantly update, 1 = show recent damage.
 !freeram_RecordEffect	= $7E0DC3	;>[1 byte] record effect of fill, not used if disabled.
 !freeram_RecHangTmr	= $7E0DC4	;>[1 byte] hang timer, how long before record decrease after damage, not used if disabled.
!TileLength		= 07		;>[decimal] how many 8x8 tiles long the bar is, minus the 2 end tiles.

;^If you change the length, CTRL+F "Scratch RAM to store bar to tiles". This does not impact the maximum
;amount of health.
;-----------------------------------------------------------;
;sfx used when the bar is filling up: (not used if disabled)
;-----------------------------------------------------------;
!sfx_filling		= 1		;>play sfx: 0 = false, 1 = true.
!ram_sound		= $1DFC		;>sound ram address for sound effect
!val_sound		= $23		;>sound number (will play when meter is filling).
;-------------------------------------------------------;
;tiles to store to HUD:
;-------------------------------------------------------;
!display_numb		= 01		;>Display the HP in numerical:
					; 00 = None, display only percentage as bar.
					; 01 = Only display the current HP.
					; 02 = Display current/max.
!val_out_of_symbol	= $48		;>the symbol "/", not used if disabled
!end_tile_TF		= 01		;>draw end tiles; 0 = false, 1 = true.
!LeftEndTile		= $38		;\end tiles, they are not effected by fill.
!RightEndTile		= $39		;/
!val_tile0_8		= $68		;0/8 (empty) tile $FC is an invisible tile.
!val_tile1_8		= $67		;1/8
!val_tile2_8		= $66		;2/8
!val_tile3_8		= $65		;...
!val_tile4_8		= $59
!val_tile5_8		= $58
!val_tile6_8		= $57
!val_tile7_8		= $56
!val_tile8_8		= $55		;...8/8 (filled)

!SBSize			= $02		;>$01 if 1 byte per 8x8 tile, $02 = 2 bytes per 8x8 tile (SSB patch).
!BossHPBarPos		= $7FA064	;>position of bar, can be smw's HUD, or SSB's ram (but even numbers only).
!BossHPCountPos		= $7FA02A	;>same as above but for numbers.

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;This is here so the table won't display glitched tiles.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
print "INIT ",pc
	RTL
print "MAIN ",pc
        PHB                     
        PHK                     
        PLB                     
        JSR ShowBossHP
        PLB                     
        RTL

ShowBossHP:
;	LDA.w $13D4		;\If player pause the game, then done.
;	BEQ +			;|(meter fills up during pause).
;	JMP .HPdone		;|
;+				;/
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Reverts status bar when 0HP, edit this if you change the HUD.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	LDA #$FC		;>load invisible blank tile

	STA !BossHPCountPos			;>current's thousands place
	STA !SBSize*1+!BossHPCountPos		;>current's hundreds place
	STA !SBSize*2+!BossHPCountPos		;>current's tens place
	STA !SBSize*3+!BossHPCountPos		;>current's ones place
	STA !SBSize*4+!BossHPCountPos		;>"/"
	STA !SBSize*5+!BossHPCountPos		;>max's thousands place
	STA !SBSize*6+!BossHPCountPos		;>max's hundreds place
	STA !SBSize*7+!BossHPCountPos		;>max's tens place
	STA !SBSize*8+!BossHPCountPos		;>max's ones place

	STA !BossHPBarPos			;\
	STA !SBSize*1+!BossHPBarPos		;|same as above but for bar tiles.
	STA !SBSize*2+!BossHPBarPos		;|If you change the length, add a new
	STA !SBSize*3+!BossHPBarPos		;|line ("STA !BossHPBarPos+Y") or remove where
	STA !SBSize*4+!BossHPBarPos		;|Y increments by 2 bytes.
	STA !SBSize*5+!BossHPBarPos		;|
	STA !SBSize*6+!BossHPBarPos		;|
	STA !SBSize*7+!BossHPBarPos		;|
	STA !SBSize*8+!BossHPBarPos		;/



;	LDA #$FC		;blank 8x8 tile area.
;	STA.w $0F09		;
;	STA.w $0F0D		;
;	STA.w $0F0E		;
;	STA.w $0F0F		;

;	LDA #$2E		;coin symbol and "X"
;	STA.w $0F10		;
;	LDA #$26		;
;	STA.w $0F11		;
;	LDA #$FC		;
;	STA.w $0F12		;

;	LDA #$3D		;the word "time"
;	STA.w $0F0A		;
;	LDA #$3E		;
;	STA.w $0F0B		;
;	LDA #$3F		;
;	STA.w $0F0C		;

;	!smwtimerpos = $0F25		;If you put this meter on
;	LDA $0F31			;top of the timer, remove
;	STA !smwtimerpos		;the semicolon (";") on the
;	LDA $0F32			;very left side.
;	STA !smwtimerpos		;
;	LDA $0F33			;
;	STA !smwtimerpos		;

;	LDA.w $0DBF			;restore coin counter routine
;	CMP #$0A
;	BCC .OneCoinDigit
;	JSR HexDec2
;	STY.w $0F13	
;	STA.w $0F14	
;	BRA .restoreSBdone
;	.OneCoinDigit
;	STA.w $0F14	
;	LDA #$FC
;	STA.w $0F13	
;	.restoreSBdone

;--------------------------------------------------------------------;
;customizable restore stuff ends here. The code below here
;is the "check if hp is gone".
;--------------------------------------------------------------------;
	REP #$20			;\if max HP is zero, then return.
	LDA !freeram_MaxHP		;|(don't divide by zero!! It would endlessly loop!)
	SEP #$20			;|
	BNE +				;/
	JMP .HPdone
+
if !UseRecordEffect != 0
	LDA !freeram_RecordEffect	;\If everything else is all zero, then hide the meter.
	BEQ +				;|
	JMP .ShowMeter			;|
endif					;|
+	LDA !freeram_CurrentHP		;|
	BNE .ShowMeter			;|
	JMP .HPdone			;/

.ShowMeter
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;This code below will display the sprite's current HP on status bar.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
if !display_numb >= 1
	REP #$20
	LDA !freeram_CurrentHP		;\if boss HP less than 10,000
	CMP #$2710			;|then show numbers
	BCC .normal_hp			;/
	LDA #$2727			;\otherwise display "----"
	STA.w $0000			;|to not show glitched HP
	STA.w $0000+2			;|counter if > 9999 and return
	SEP #$20			;/
	JMP .HPdone
.normal_hp
	LDA !freeram_CurrentHP		;these are digit display (converting to decimal).
	JSR HexDec16to311			
	STY.w $0000+2			;>3rd digit (tens place)
	STX.w $0000+3			;>4th digit (ones place)
	SEP #$20
	JSR HexDec2
	STA.w $0000+1			;>2nd digit (hundreds place)
	STY.w $0000			;>1st digit (thousands place)
	REP #$20
	LDA !freeram_CurrentHP		;\if less than 10, one digit
	CMP #$000A			;|
	BCC .onedigit			;/
	CMP #$0064			;\if less than 100, two digit
	BCC .twodigit			;/
	CMP #$03E8			;\if less than 1000, three digit
	BCC .threedigit			;/
	BRA .dont_hide_digit		;>4 digits = don't hide any digits.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Hide current HP digits if leading zero
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.onedigit			;\hide unessary "0" digits.
	SEP #$20		;|
	LDA #$FC		;|\hide 1st to 3rd digit
	STA.w $0000		;||
	STA.w $0000+1		;||
	STA.w $0000+2		;||
	BRA +			;|/
.twodigit			;|
	SEP #$20		;|
	LDA #$FC		;|\hide 1st to 2nd digit
	STA.w $0000		;||
	STA.w $0000+1		;||
	SEP #$20		;||
	BRA +			;|/
.threedigit			;|
	SEP #$20		;|\hide only 1st digit
	LDA #$FC		;||
	STA.w $0000		;||
	BRA +			;|/
.dont_hide_digit		;|
	SEP #$20		;/
+
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Scratch RAM write current HP:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	LDA $0000				;\\first digit (thousands place)
	STA !BossHPCountPos			;|/
	LDA $0001				;|\second digit (hundreds place)
	STA !SBSize*1+!BossHPCountPos		;|/
	LDA $0002				;|\third digit (tens place)
	STA !SBSize*2+!BossHPCountPos		;|/
	LDA $0003				;|\fourth digit (ones place)
	STA !SBSize*3+!BossHPCountPos		;//
endif				;>leave this here!
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;This code below will display the sprite's max HP on status bar.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
if !display_numb == 2
	LDA #!val_out_of_symbol
	STA.w $0000+4		;>"/" symbol
	REP #$20
	LDA !freeram_MaxHP
	JSR HexDec16to311
	STY.w $0000+7		;>3rd digit (tens place)
	STX.w $0000+8		;>4th digit (ones place)
	SEP #$20
	JSR HexDec2
	STA.w $0000+6		;>2nd digit (hundreds place)
	STY.w $0000+5		;>1st digit (thousands place)
	REP #$20
	LDA !freeram_MaxHP	;\if less than 10, one digit
	CMP #$000A		;|
	BCC .onedigit1		;/
	CMP #$0064		;\if less than 100, two digit
	BCC .twodigit1		;/
	CMP #$03E8		;\if less than 1000, three digit
	BCC .threedigit1	;/
	BRA .dont_hide_digit1	;>4 digits = don't hide any digits.
.onedigit1			;\hide unessary "0" digits.
	SEP #$20		;|
	LDA #$FC		;|
	STA.w $0000+5		;|
	STA.w $0000+6		;|
	STA.w $0000+7		;|
	BRA +			;|
.twodigit1			;|
	SEP #$20		;|
	LDA #$FC		;|
	STA.w $0000+5		;|
	STA.w $0000+6		;|
	BRA +			;|
.threedigit1			;|
	SEP #$20		;|
	LDA #$FC		;|
	STA.w $0000+5		;|
	BRA +			;|
.dont_hide_digit1		;|
	SEP #$20		;/
+
;;;;;;;;;;;;;;;;;;;;;;;;;;
;Scratch RAM write max HP:
;;;;;;;;;;;;;;;;;;;;;;;;;;
	LDA $0004				;\the "/" symbol
	STA !SBSize*4+!BossHPCountPos		;/
	LDA $0005				;\\first digit (thousands place)
	STA !SBSize*5+!BossHPCountPos		;|/
	LDA $0006				;|\second digit (hundreds place)
	STA !SBSize*6+!BossHPCountPos		;|/
	LDA $0007				;|\third digit (tens place)
	STA !SBSize*7+!BossHPCountPos		;|/
	LDA $0008				;|\fourth digit (ones place)
	STA !SBSize*8+!BossHPCountPos		;//
endif					;>leave this "endif" here!

;------------------------------------------------------------------------
;CurrentHP/MaxHP -> Pixels/MaxPixels percentage bar conversion routine.
;If you are using sa-1, it will instead use sa-1's built-in multiplication
;and division register instead of this.
;------------------------------------------------------------------------
	REP #$20		;\If the boss has at least 1HP, then calculate
	LDA !freeram_CurrentHP	;|
	BNE .non_emptybar	;/
	SEP #$20		;\else if the boss has 0HP, then skip calculations
	STZ $00			;|
	JMP .HPPercentageDone	;/
.non_emptybar			;>haven't SEP #$20 yet.
;---------------------------------------
	;Non-sa-1 smw:
;multiply
	LDA !freeram_CurrentHP	;\current_HP * max_number_of_pixels.. 
	STA $00			;|$7E0000 * $7E0002 = A
	LDA.w #!TileLength*8	;|>convert tile length pixels 1-byte value to 2-bytes
	STA $02			;| so it calculates correctly
	STA $04			;|
-	LSR $02			;|
	BEQ .end		;|
	BCC +			;|
	CLC			;|
	ADC $00			;|
+	ASL $00			;|
	BRA -			;|
.end				;|
	CLC			;|
	ADC $00			;|
	SEC			;|\remove 1-extra multiplication.
	SBC $04			;//
;divide
	STA $00				;\...divide by maxHP.
	LDA !freeram_MaxHP		;|$7E0000 / $7E0002= ...
	STA $02				;|$7E0004 = Quotient 
	PHB				;|$7E0006 = Remainder 
	PHK				;|Y = 1 if division by zero
	PLB				;|
	JSR Divide			;|
	PLB				;/

	LDX $04				;>Don't need to load quotient twice for each condition.
	LDA !freeram_MaxHP		;\Half of max
	LSR				;/
	CMP $06				;>compare with remainder
	BCS +				;>if half of max is bigger than remainder (or remainder is smaller), then round down
	INX				;\Round quotient half up.
	STX $00				;/
	SEP #$20
	BRA .HPPercentageDone
;RoundDown
+	STX $00				;>Store round down result to input for amount of pixels.
	SEP #$20

if !UseRecordEffect != 0
	.HPPercentageDone
	;-----------------------------------------------------------------------
	;This the record effect, it displays the translucent fill part to
	;indicate the recent amount of damage and gradually fills if HP is
	;higher than before (when the boss heals or battle starts).
	;-----------------------------------------------------------------------
		LDA $00				;>load current HP in %...
		CMP !freeram_RecordEffect	;>compare with record...
		BEQ .SetTimer			;>If HP % is = record %, then start timer every frame and draw.
		BCC .TransparentDmg		;>If HP % is < record %, then show translucent part.

	;FillingAnimation:			;>Otherwise HP % is > record %, then fill up the meter.
		LDA !freeram_RecordEffect	;\Fill the bar up.
		STA $00				;|
		INC A				;|
		STA !freeram_RecordEffect	;/
		if !sfx_filling != 0
			LDA $13				;\sfx when bar filling
			AND #$03			;|
			BNE .DrawMeter			;|
			LDA #!val_sound			;|
			STA !ram_sound			;/
		endif
	.SetTimer
		LDA.b #90
		STA !freeram_RecHangTmr
		JMP .DrawMeter

	.TransparentDmg
		LDA !freeram_RecHangTmr		;\What timer behavors do.
		BEQ +				;|
		DEC				;|
		STA !freeram_RecHangTmr		;/
	+
		LDA $13				;\If the rightmost bit (0 bit) is clear, then
		AND #$01			;|show the record effect, otherwise show current %
		BNE .DrawMeter			;/the current HP % (flicker as pseudo see-through).
		LDA $13				;\If 2nd frame, then decrease translucent.
		AND #$01			;|
		BEQ .DecrementHangTimer		;/
	.RecordFill
		LDA !freeram_RecordEffect	;\Else write the record fill instead of current.
		JMP .SetRecord			;/

	.DecrementHangTimer
		LDA !freeram_RecHangTmr		;\If still time left, keep hanging
		BNE .RecordFill			;/
		LDA !freeram_RecordEffect	;\Decrement and then store to fill data.
		DEC A				;|
		STA !freeram_RecordEffect	;/
	.SetRecord
		STA $00				;>store record effect data to scratch for the amount of fill.
else
	.HPPercentageDone
endif
.DrawMeter
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;draw bar (note that this is much faster than before, doesn't have to draw
;empty tile that is going to be replaced with a filled tile, making the
;routine that draws empty pointless).
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Draw full 8x8 tiles:
+	LDA $00			;>How much fill in bar (in pixels; each 8x8 tile's maximum is 8).
	LSR #$03		;>amount of fill divide by 8 (rounded down), being how many full 8x8 tiles.
	CMP.b #!TileLength	;\Prevent buffer overflow when number of full 8x8
	BCC .NotFull		;/tiles takes !TileLength amount of bytes.
;.Full
	LDA.b #!TileLength	;>Load the maximum 8x8s in the fillable space.
	JSR WriteFull8x8s	;>Write full 8x8s on ALL fillable space.
	BRA .NoEmptyTile	;>And skip the fraction tile and 0/8 tile drawing routine.
.NotFull
	JSR WriteFull8x8s

.FractionTile
;Write fraction tile:
	LDA $00			;\Modulo 8 for tile tables.
	AND #$07		;/
	TAX			;>Transfer fraction tiles to X
	LDA .fraction_tiles,x	;>Load fraction tiles (loops within 0-7)
	STA.w $0001,y		;>Store on position depending on amount of full 8x8 tiles

;Write empty tiles after fraction:
	INY			;>the first empty tile after the fraction tile.
	CPY #!TileLength	;\If empty about to be located past the last fillable 8x8,
	BCS .NoEmptyTile	;/don't write empty on +1 extra index.
-	LDA #!val_tile0_8	;\write empty tiles after the fraction tile.
	STA.w $0001,y		;|
	INY			;|
	CPY.b #!TileLength	;/
	BCC -
.NoEmptyTile
;----------------------------------------------------------------------------------------------------------------------
;Scratch ram to store bar to tiles. Do not use $0000, start at $0001 instead. Because
;$0000 reserved for the amount of fill pixels during the "fill display" routine. Changing
;the bar length means you need to add/delete items from the bottom.
;
;	LDA.w $XXXX			;>scratch ram holding tile number, keep +1 byte for each tile
;	STA !SBSize*Y+!BossHPBarPos	;>store to advance super status bar. Y keeps incrementing up by 1 each new tile
;----------------------------------------------------------------------------------------------------------------------
	if !end_tile_TF != 0
		LDA #!LeftEndTile	;\Store left end tile
		STA !BossHPBarPos	;/
	endif
	LDA.w $0001			;\use saved tile data in scratch ram and copy them to status bar tiles.
	STA !SBSize*1+!BossHPBarPos	;|(these are fillable tiles)
	LDA.w $0002			;|
	STA !SBSize*2+!BossHPBarPos	;|
	LDA.w $0003			;|
	STA !SBSize*3+!BossHPBarPos	;|
	LDA.w $0004			;|
	STA !SBSize*4+!BossHPBarPos	;|
	LDA.w $0005			;|
	STA !SBSize*5+!BossHPBarPos	;|
	LDA.w $0006			;|
	STA !SBSize*6+!BossHPBarPos	;|
	LDA.w $0007			;|
	STA !SBSize*7+!BossHPBarPos	;/
	if !end_tile_TF != 0
		LDA #!RightEndTile	;\Store right end tile
		STA !BossHPBarPos+16	;/(do note that you still have to add by 2 after the last fillable tile)
	endif
;-----------------------
;Do not change this!!
;-----------------------
.HPdone
	RTS

;-------------------------------------------------------------------------;
;This is the table to display 0-7 different tiles, do not change this!	  ;
;-------------------------------------------------------------------------;
.fraction_tiles
	db !val_tile0_8,!val_tile1_8,!val_tile2_8,!val_tile3_8,!val_tile4_8,!val_tile5_8,!val_tile6_8,!val_tile7_8
;-------------------------------------------------;
;these are recycled routines, do not change.      ;
;-------------------------------------------------;
;Write full 8x8 tiles (placed before fraction)
;Both used when the bar is at least 8 pxls
;and if the bar is 100% filled.
;Input and output: Y=how many full 8x8 tiles.
WriteFull8x8s:
	TAY			;>Transfer number of full 8x8s to Y
	BEQ +			;>If there are no full 8x8 tile, skip.
	PHY			;>Save Y since after the following loop, needs the Y back.
	DEY			;>because index 0 exist (index value 0 is *AT* $7E0001).
-	LDA #!val_tile8_8	;\write full tiles that is before
	STA.w $0001,y		;/fraction tiles.
	DEY			;>Next index 
	BPL -			;>if zero or positive, loop and repeat drawing full 8x8s
	PLY			;>Get number of full 8x8s back into Y
+
	RTS
;hex-decimal converter------------------------
HexDec16to311:
	STA $4204
	LDY #$0A
	STY $4206
	JSR .Waste24Cycles
	LDX $4216
	LDA $4214
	STA $4204
	LDY #$0A
	STY $4206
	JSR .Waste24Cycles
	LDY $4216
	LDA $4214
	RTS
.Waste24Cycles
	JSR .Return
.Return
	RTS

HexDec2:
	LDY #$00	;\HexDec for the first-two digits
.loops			;|(hundreds and thousands place)
	CMP #$0A	;|
	BCC .outloop	;|
	SBC #$0A	;|
	INY		;|
	BRA .loops	;|
.outloop		;|
	RTS		;/
;Division subroutine------------------------------
Divide:
	LDY #$00
	STZ $04
	STZ $06
	LDA $02
	BNE .Continue
	INY
	RTS
.Continue
	LDX #$0F
.Loop				; \
	CPX #$00		; | For i=15...0
	BMI .EndLoop		; /

	LDA $06			; \
	ASL A			; | R <<=1
	STA $06			; /

	LDA $00
	JSR .NewGetBit
	ORA $06
	STA $06

	LDA $06
	CMP $02	
	BCC .Next
	
	LDA $06
	SEC
	SBC $02
	STA $06

	;LDA.w #$0001
	JSR .NewMultiplyBit
	ORA $04
	STA $04
.Next
	DEX
	JMP .Loop
.EndLoop
	LDY.b #$00
	RTS
.NewGetBit
	PHX
	PHY
	LDY.b #$00
	PHA
	TXA 
	ASL
	AND.w #$00FF
	TAX
	PLA
	AND.w Look_Table,x
	BEQ +
	INY
+
	TYA
	PLY
	PLX
	RTS
.NewMultiplyBit
	PHX
	TXA
	ASL
	AND.w #$00FF
	TAX
	LDA Look_Table,x
	PLX
	RTS

Look_Table:
	dw $0001,$0002,$0004,$0008,$0010,$0020,$0040,$0080
	dw $0100,$0200,$0400,$0800,$1000,$2000,$4000,$8000