;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; This is an asar uberasm version of the "graphical_hp_bar.asm". To use it in the level, just put this:
;; "JSR ShowBossHP" and follow the instructions below marked with rows of ~ to insert the code.
;; This code is sa-1 hybrid (uberasm main patch should hold the defines values across incsrc files).
;;
;; This will display HP meter as graphical (as a rectangular bar with a fill). HP is based
;; on the number of pixels (either by 1 HP per pixel, or 256 HP per pixel).
;;
;; Notes:
;;
;; -The tile properties (YXPCCCTT) of the layer 3 status bar/HUD also effects the tiles displayed by
;;  this code (so if a tile is flipped, a bar or counter placed on that tile gets flipped).
;;
;; -The score area on the status bar stores tiles every frame. If you use that space, parts of the
;;  display will get overwritten by the score tiles due to the sequence of the game runs. Using
;;  Iceguy's "score disable" will not fix this problem, so in the ROM map, change pc address 0x010DB
;;  (snes: $00:8EDB) and 0x01109 (snes: $00:8F09) from [20 12 90] to [EA EA EA] to disable writing the
;;  score onto the status bar
;;
;; -To change the HP to use one-byte, simply set the number in "!two_byteHP" to 0
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Put these defines at the top in main uberasm patch ("asar_patch.asm")
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!freeram_CurrentHP	= $7E0060	;>[1 or 2 bytes], the sprite's current HP.
!UseRecordEffect	= 01		;>0 = instantly update, 1 = show recent damage as transperent.
 !freeram_RecordEffect	= $7E0DC3	;>[1 byte] record effect of fill, not used if disabled.
 !freeram_RecHangTmr	= $7E0DC4	;>[1 byte] hang timer, how long before record decrease after damage, not used if disabled.
!two_byteHP		= 1		;>!freeram_CurrentHP is:
					; 0 = 1-byte; 1HP per pixel.
					; 1 = 2-bytes; 256HP per pixel.
!TileLength		= 07		;>[decimal] how many 8x8 tiles long the bar is, minus the 2 end tiles.

;^If you change the length, CTRL+F "Scratch RAM to store bar to tiles". Each new 8x8 tile adds
;256 or 2048 to the maximum HP.
;-------------------------------------------------------;
;sfx used when the bar is filling up:
;-------------------------------------------------------;
!sfx_filling		= 01		;>play sound if meter filling up; 0 = false,
					; 1 = true. Only happens if using the record
					; effect.
!ram_sound		= $1DFC		;>sound ram address for sound effect (ram address automatically be converted to sa-1 if sa-1 rom).
!val_sound		= $23		;>sound number (will play when meter is
					; filling).
;-------------------------------------------------------;
;tiles to store to HUD:
;-------------------------------------------------------;
!end_tile_TF		= 01		;>draw end tiles; 0 = false, 1 = true.
!LeftEndTile		= $38		;\end tiles, they are not effected by fill.
!RightEndTile		= $39		;/
!val_tile0_8		= $68		;0/8 (empty) tile $FC is an invisible tile.
!val_tile1_8		= $67		;1/8
!val_tile2_8		= $66		;2/8
!val_tile3_8		= $65		;...
!val_tile4_8		= $59
!val_tile5_8		= $58
!val_tile6_8		= $57
!val_tile7_8		= $56
!val_tile8_8		= $55		;...8/8 (full tile)

!SBSize			= $02		;>$01 if 1 byte per 8x8 tile, $02 = 2 bytes per 8x8 tile (SSB patch).
!BossHPBarPos		= $7FA064	;>position of bar, can be smw's HUD, or SSB's ram.

;^Do note that position is not automatically converted to sa-1, because its also used for the
;super status bar patch's freeram. Therefore, if you're using the old HUD (or 1 byte per 8x8),
;and use sa-1, use $6EF9 instead.
;--------------------------
;DO NOT CHANGE THIS BELOW!
;--------------------------

if !two_byteHP == 0
	!CurrentHPPxls = !freeram_CurrentHP
else
	!CurrentHPPxls = !freeram_CurrentHP+1	;>high byte is the amount of fill pixels
endif
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Put this entire code on the very bottom in the uberasm's level code file.
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

ShowBossHP:
	PHB			;\prevent tile table from glitching up.
	PHK			;|
	PLB			;/

	LDA.w $13D4+!Base2	;\If player pause the game, then done.
	BEQ +			;|(meter fills up during pause).
	JMP .CompletelyDone	;|
+				;/
;--------------------------------------------------------------------------------
;This code restores the status bar when the meter is empty, so if you have edited
;the status bar, or move this lifebar position, you should change this also.
;--------------------------------------------------------------------------------
	LDA #$FC		;\revert new status bar to normal; Change this if
	STA !BossHPBarPos	;|you have changed the status bar tiles,
	STA !BossHPBarPos+2	;|move and/or move this health meter.
	STA !BossHPBarPos+4	;|
	STA !BossHPBarPos+6	;|
	STA !BossHPBarPos+8	;|
	STA !BossHPBarPos+10	;|
	STA !BossHPBarPos+12	;|
	STA !BossHPBarPos+14	;|
	STA !BossHPBarPos+16	;/


;	LDA #$FC	;blank 8x8 tile area.
;	STA $0F09+!Base2	;
;	STA $0F0D+!Base2	;
;	STA $0F0E+!Base2	;
;	STA $0F0F+!Base2	;

;	LDA #$2E		;coin symbol and "X"
;	STA $0F10+!Base2	;
;	LDA #$26		;
;	STA $0F11+!Base2	;
;	LDA #$FC		;
;	STA $0F12+!Base2	;

;	LDA #$3D		;the word "time"
;	STA $0F0A+!Base2	;
;	LDA #$3E		;
;	STA $0F0B+!Base2	;
;	LDA #$3F		;
;	STA $0F0C+!Base2	;

;	!smwtimerpos = $0F25+!Base2	;If you put this meter on
;	LDA $0F31+!Base2		;top of the timer, remove
;	STA !smwtimerpos		;the semicolon (";") on the
;	LDA $0F32+!Base2		;very left side.
;	STA !smwtimerpos		;
;	LDA $0F33+!Base2		;
;	STA !smwtimerpos		;

;	LDA $0DBF+!Base2		;restore coin counter routine
;	CMP #$0A
;	BCC .OneCoinDigit
;	JSR .hex_dec_convert
;	STY $0F13+!Base2
;	STA $0F14+!Base2
;	BRA .restoreSBdone
;	.OneCoinDigit
;	STA $0F14+!Base2
;	LDA #$FC
;	STA $0F13+!Base2
;	.restoreSBdone
;--------------------------------------------------------------------;
;customizable restore stuff ends here. The code below here
;is the "check if hp is gone".
;--------------------------------------------------------------------;
if !two_byteHP == 0
	LDA !freeram_CurrentHP
	if !UseRecordEffect == 0
		ORA !freeram_RecordEffect
	endif
	BNE .HPNotEmpty
	JMP .CompletelyDone
else			;for 2-bytes hp
	REP #$20
	LDA !freeram_CurrentHP
	SEP #$20
	if !UseRecordEffect != 0
		BEQ +
		JMP .HPNotEmpty
+		LDA !freeram_RecordEffect
		BNE .HPNotEmpty
		JMP .CompletelyDone
	else
		BNE .HPNotEmpty
		JMP .CompletelyDone
	endif
endif
.HPNotEmpty
	LDA !CurrentHPPxls		;\set current hp's high byte to
	STA $00				;/current and transperent parts for the bar.
if !two_byteHP != 0
	LDA !freeram_CurrentHP		;\If low byte is #$00 to #$7F inclusive,
	BPL .NoRoundUp			;/then skip round half up the high byte (that is used for amount of pixels)
	LDA $00				;\increment high byte for the approximate pixels.
	INC A				;|
	STA $00				;/
.NoRoundUp
endif
if !UseRecordEffect != 0
	;-----------------------------------------------------------------------
	;This the record effect, it displays the translucent fill part to
	;indicate the recent amount of damage and gradually fills if HP is
	;higher than before (when the boss heals or battle starts).
	;-----------------------------------------------------------------------
		LDA $00				;>Amount of pixels...
		CMP !freeram_RecordEffect	;>compare with record effect...
		BEQ .SetTimer			;>If HP % is = record %, then start timer every frame and draw.
		BCC .TransparentDmg		;>If HP is < record, then show transperent part.
	
	;FillingAnimation:			;>Otherwise HP is > record, then fill up the meter.
		LDA !freeram_RecordEffect	;\Fill the bar up.
		STA $00				;|
		INC A				;|
		STA !freeram_RecordEffect	;/
		if !sfx_filling != 0
			LDA $13				;\sfx when bar filling
			AND #$03			;|
			BNE .DrawMeter			;|
			LDA #!val_sound			;|
			STA !ram_sound			;/
		endif
	.SetTimer
		LDA.b #90
		STA !freeram_RecHangTmr
		JMP .DrawMeter

	.TransparentDmg
		LDA !freeram_RecHangTmr		;\What timer behavors do.
		BEQ +				;|
		DEC				;|
		STA !freeram_RecHangTmr		;/
	+
		LDA $13				;\If the rightmost bit (0 bit) is clear, then
		AND #$01			;|show the record effect, otherwise show
		BNE .DrawMeter			;/the current HP %.
		LDA $13				;\If 2nd frame, then decrease translucent.
		AND #$01			;|
		BEQ .DecrementHangTimer		;/
	.RecordFill
		LDA !freeram_RecordEffect	;\Else write the record fill instead of current.
		JMP .SetRecord			;/

	.DecrementHangTimer
		LDA !freeram_RecHangTmr		;\If still time left, keep hanging
		BNE .RecordFill			;/
		LDA !freeram_RecordEffect	;\Decrement and then store to fill data.
		DEC A				;|
		STA !freeram_RecordEffect	;/
	.SetRecord
		STA $00				;>store record effect data to scratch for the amount of fill.
else
	.HPPercentageDone
endif
.DrawMeter
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;draw bar (note that this is much faster than before, doesn't have to draw
;empty tile that is going to be replaced with a filled tile, making the
;routine that draws empty pointless).
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Draw full 8x8 tiles:
+	LDA $00			;>How much fill in bar (in pixels; each 8x8 tile's maximum is 8).
	LSR #$03		;>amount of fill divide by 8 (rounded down), being how many full 8x8 tiles.
	CMP.b #!TileLength	;\Prevent buffer overflow when number of full 8x8
	BCC .NotFull		;/tiles takes !TileLength amount of bytes.
;.Full
	LDA.b #!TileLength	;>Load the maximum 8x8s in the fillable space.
	JSR WriteFull8x8s	;>Write full 8x8s on ALL fillable space.
	BRA .NoEmptyTile	;>And skip the fraction tile and 0/8 tile drawing routine.
.NotFull
	JSR WriteFull8x8s

.FractionTile
;Write fraction tile:
	LDA $00			;\Modulo 8 for tile tables.
	AND #$07		;/
	TAX			;>Transfer fraction tiles to X
	LDA .fraction_tiles,x	;>Load fraction tiles (loops within 0-7)
	STA.w $0001+!Base1,y	;>Store on position depending on amount of full 8x8 tiles

;Write empty tiles after fraction:
	INY			;>the first empty tile after the fraction tile.
	CPY #!TileLength	;\If empty about to be located past the last fillable 8x8,
	BCS .NoEmptyTile	;/don't write empty on +1 extra index.
-	LDA #!val_tile0_8	;\write empty tiles after the fraction tile.
	STA.w $0001+!Base1,y	;|
	INY			;|
	CPY.b #!TileLength	;/
	BCC -
.NoEmptyTile
;----------------------------------------------------------------------------------------------------------------------
;Scratch ram to store bar to tiles. Do not use $0000, start at $0001 instead. Because
;$0000 reserved for the amount of fill pixels during the "fill display" routine. Changing
;the bar length means you need to add/delete items from the bottom.
;
;	LDA.w $XXXX+!Base1		;>scratch ram holding tile number, keep +1 byte for each tile
;	STA !SBSize*Y+!BossHPBarPos	;>store to advance super status bar. Y keeps incrementing up by 1 each new tile
;----------------------------------------------------------------------------------------------------------------------
	LDA #!LeftEndTile	;\Store left end tile
	STA !BossHPBarPos	;/

	LDA.w $0001+!Base1		;\use saved tile data in scratch ram and copy them to status bar tiles.
	STA !SBSize*1+!BossHPBarPos	;|(these are fillable tiles)
	LDA.w $0002+!Base1		;|
	STA !SBSize*2+!BossHPBarPos	;|
	LDA.w $0003+!Base1		;|
	STA !SBSize*3+!BossHPBarPos	;|
	LDA.w $0004+!Base1		;|
	STA !SBSize*4+!BossHPBarPos	;|
	LDA.w $0005+!Base1		;|
	STA !SBSize*5+!BossHPBarPos	;|
	LDA.w $0006+!Base1		;|
	STA !SBSize*6+!BossHPBarPos	;|
	LDA.w $0007+!Base1		;|
	STA !SBSize*7+!BossHPBarPos	;/

	LDA #!RightEndTile	;\Store right end tile
	STA !BossHPBarPos+16	;/(do note that you still have to add by 2 after the last fillable tile)
;-----------------------
;Do not change this!!
;-----------------------
.CompletelyDone
        PLB
	RTS
;-------------------------------------------------------------------------;
;This is the table to display 0-7 different tiles, do not change this!	  ;
;-------------------------------------------------------------------------;
.fraction_tiles
	db !val_tile0_8,!val_tile1_8,!val_tile2_8,!val_tile3_8,!val_tile4_8,!val_tile5_8,!val_tile6_8,!val_tile7_8
;-------------------------------------------------;
;these are recycled routines, do not change.      ;
;-------------------------------------------------;
;Write full 8x8 tiles (placed before fraction)
;Both used when the bar is at least 8 pxls
;and if the bar is 100% filled.
;Input and output: Y=how many full 8x8 tiles.
WriteFull8x8s:
	TAY			;>Transfer number of full 8x8s to Y
	BEQ +			;>If there are no full 8x8 tile, skip.
	PHY			;>Save Y since after the following loop, needs the Y back.
	DEY			;>because index 0 exist (index value 0 is *AT* $7E0001).
-	LDA #!val_tile8_8	;\write full tiles that is before
	STA.w $0001+!Base1,y	;/fraction tiles.
	DEY			;>Next index 
	BPL -			;>if zero or positive, loop and repeat drawing full 8x8s
	PLY			;>Get number of full 8x8s back into Y
+
	RTS
;hex-decimal converter------------------------
;Needed to restore smw status bar's numbers.
.hex_dec_convert
	LDY #$00
.base10loop
	CMP #$0A
	BCC .decimal_done
	SBC #$0A
	INY
	BRA .base10loop
.decimal_done
	RTS